@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -46,7 +46,9 @@ static void print_segment(Segments *segments,index_t item);
 static void print_way(Ways *ways,index_t item);
 static void print_turnrelation(Relations *relations,index_t item,Segments *segments,Nodes *nodes);
 
-static void print_head_osm(void);
+static void print_head_osm(int coordcount,double latmin,double latmax,double lonmin,double lonmax);
+static void print_region_osm(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,
+                             double latmin,double latmax,double lonmin,double lonmax,int option_no_super);
 static void print_node_osm(Nodes *nodes,index_t item);
 static void print_segment_osm(Segments *segments,index_t item,Ways *ways);
 static void print_turnrelation_osm(Relations *relations,index_t item,Segments *segments,Nodes *nodes);
@@ -136,6 +138,9 @@ int main(int argc,char** argv)
 
  if(option_visualiser)
    {
+    Highway highway;
+    Transport transport;
+
     if(coordcount!=4)
        print_usage(0,NULL,"The --visualiser option must have --latmin, --latmax, --lonmin, --lonmax.\n");
 
@@ -148,6 +153,10 @@ int main(int argc,char** argv)
        OutputSuper(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"oneway"))
        OutputOneway(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
+    else if(!strncmp(option_data,"highway",7) && option_data[7]=='-' && (highway=HighwayType(option_data+8))!=Way_Count)
+       OutputHighway(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax,highway);
+    else if(!strncmp(option_data,"transport",9) && option_data[9]=='-' && (transport=TransportType(option_data+10))!=Transport_None)
+       OutputTransport(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax,transport);
     else if(!strcmp(option_data,"turns"))
        OutputTurnRestrictions(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"speed"))
@@ -333,73 +342,10 @@ int main(int argc,char** argv)
     if(coordcount>0 && coordcount!=4)
        print_usage(0,NULL,"The --dump-osm option must have all of --latmin, --latmax, --lonmin, --lonmax or none.\n");
 
-    print_head_osm();
+    print_head_osm(coordcount,latmin,latmax,lonmin,lonmax);
 
     if(coordcount)
-      {
-       ll_bin_t latminbin=latlong_to_bin(radians_to_latlong(latmin))-OSMNodes->file.latzero;
-       ll_bin_t latmaxbin=latlong_to_bin(radians_to_latlong(latmax))-OSMNodes->file.latzero;
-       ll_bin_t lonminbin=latlong_to_bin(radians_to_latlong(lonmin))-OSMNodes->file.lonzero;
-       ll_bin_t lonmaxbin=latlong_to_bin(radians_to_latlong(lonmax))-OSMNodes->file.lonzero;
-       ll_bin_t latb,lonb;
-       index_t item,index1,index2;
-
-       if(latminbin<0)                      latminbin=0;
-       if(latmaxbin>OSMNodes->file.latbins) latmaxbin=OSMNodes->file.latbins-1;
-       if(lonminbin<0)                      lonminbin=0;
-       if(lonmaxbin>OSMNodes->file.lonbins) lonmaxbin=OSMNodes->file.lonbins-1;
-
-       /* Loop through all of the nodes. */
-
-       for(latb=latminbin;latb<=latmaxbin;latb++)
-          for(lonb=lonminbin;lonb<=lonmaxbin;lonb++)
-            {
-             ll_bin2_t llbin=lonb*OSMNodes->file.latbins+latb;
-
-             if(llbin<0 || llbin>(OSMNodes->file.latbins*OSMNodes->file.lonbins))
-                continue;
-
-             index1=LookupNodeOffset(OSMNodes,llbin);
-             index2=LookupNodeOffset(OSMNodes,llbin+1);
-
-             for(item=index1;item<index2;item++)
-               {
-                Node *node=LookupNode(OSMNodes,item,1);
-                double lat=latlong_to_radians(bin_to_latlong(OSMNodes->file.latzero+latb)+off_to_latlong(node->latoffset));
-                double lon=latlong_to_radians(bin_to_latlong(OSMNodes->file.lonzero+lonb)+off_to_latlong(node->lonoffset));
-
-                if(lat>latmin && lat<latmax && lon>lonmin && lon<lonmax)
-                  {
-                   Segment *segment;
-
-                   print_node_osm(OSMNodes,item);
-
-                   segment=FirstSegment(OSMSegments,OSMNodes,item,1);
-
-                   while(segment)
-                     {
-                      if(item>OtherNode(segment,item))
-                         if(!option_no_super || IsNormalSegment(segment))
-                            print_segment_osm(OSMSegments,IndexSegment(OSMSegments,segment),OSMWays);
-
-                      segment=NextSegment(OSMSegments,segment,item);
-                     }
-
-                   if(IsTurnRestrictedNode(node))
-                     {
-                      index_t relindex=FindFirstTurnRelation1(OSMRelations,item);
-
-                      while(relindex!=NO_RELATION)
-                        {
-                         print_turnrelation_osm(OSMRelations,relindex,OSMSegments,OSMNodes);
-
-                         relindex=FindNextTurnRelation1(OSMRelations,relindex);
-                        }
-                     }
-                  }
-               }
-            }
-      }
+       print_region_osm(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax,option_no_super);
     else
       {
        index_t item;
@@ -490,7 +436,7 @@ static void print_way(Ways *ways,index_t item)
  printf("Way %"Pindex_t"\n",item);
  if(*name)
     printf("  name=%s\n",name);
- printf("  type=%02x (%s%s)\n",way->type,HighwayName(HIGHWAY(way->type)),way->type&Way_OneWay?",One-Way":"");
+ printf("  type=%02x (%s%s%s)\n",way->type,HighwayName(HIGHWAY(way->type)),way->type&Way_OneWay?",One-Way":"",way->type&Way_Roundabout?",Roundabout":"");
  printf("  allow=%02x (%s)\n",way->allow,AllowedNameList(way->allow));
  if(way->props)
     printf("  props=%02x (%s)\n",way->props,PropertiesNameList(way->props));
@@ -521,12 +467,13 @@ static void print_way(Ways *ways,index_t item)
 
 static void print_turnrelation(Relations *relations,index_t item,Segments *segments,Nodes *nodes)
 {
- TurnRelation *relation=LookupTurnRelation(relations,item,1);
  Segment *segment;
+ TurnRelation *relation=LookupTurnRelation(relations,item,1);
+ Node *node=LookupNode(nodes,relation->via,1);
  index_t from_way=NO_WAY,to_way=NO_WAY;
  index_t from_node=NO_NODE,to_node=NO_NODE;
 
- segment=FirstSegment(segments,nodes,relation->via,1);
+ segment=FirstSegment(segments,node,1);
 
  do
    {
@@ -559,12 +506,122 @@ static void print_turnrelation(Relations *relations,index_t item,Segments *segme
 
 /*++++++++++++++++++++++++++++++++++++++
   Print out a header in OSM XML format.
+
+  int coordcount If true then include a bounding box.
+
+  double latmin The minimum latitude.
+
+  double latmax The maximum latitude.
+
+  double lonmin The minimum longitude.
+
+  double lonmax The maximum longitude.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void print_head_osm(void)
+static void print_head_osm(int coordcount,double latmin,double latmax,double lonmin,double lonmax)
 {
  printf("<?xml version='1.0' encoding='UTF-8'?>\n");
  printf("<osm version='0.6' generator='Routino'>\n");
+
+ if(coordcount)
+    printf("  <bounds minlat='%.6f' maxlat='%.6f' minlon='%.6f' maxlon='%.6f' />\n",
+           radians_to_degrees(latmin),radians_to_degrees(latmax),radians_to_degrees(lonmin),radians_to_degrees(lonmax));
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Print a region of the database in OSM XML format.
+
+  Nodes *nodes The set of nodes to use.
+
+  Segments *segments The set of segments to use.
+
+  Ways *ways The set of ways to use.
+
+  Relations *relations The set of relations to use.
+
+  double latmin The minimum latitude.
+
+  double latmax The maximum latitude.
+
+  double lonmin The minimum longitude.
+
+  double lonmax The maximum longitude.
+
+  int option_no_super The option to print no super-segments.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void print_region_osm(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,
+                             double latmin,double latmax,double lonmin,double lonmax,int option_no_super)
+{
+ ll_bin_t latminbin=latlong_to_bin(radians_to_latlong(latmin))-nodes->file.latzero;
+ ll_bin_t latmaxbin=latlong_to_bin(radians_to_latlong(latmax))-nodes->file.latzero;
+ ll_bin_t lonminbin=latlong_to_bin(radians_to_latlong(lonmin))-nodes->file.lonzero;
+ ll_bin_t lonmaxbin=latlong_to_bin(radians_to_latlong(lonmax))-nodes->file.lonzero;
+ ll_bin_t latb,lonb;
+ index_t item,index1,index2;
+
+ if(latminbin<0)                   latminbin=0;
+ if(latmaxbin>nodes->file.latbins) latmaxbin=nodes->file.latbins-1;
+ if(lonminbin<0)                   lonminbin=0;
+ if(lonmaxbin>nodes->file.lonbins) lonmaxbin=nodes->file.lonbins-1;
+
+ /* Loop through all of the nodes. */
+
+ for(latb=latminbin;latb<=latmaxbin;latb++)
+    for(lonb=lonminbin;lonb<=lonmaxbin;lonb++)
+      {
+       ll_bin2_t llbin=lonb*nodes->file.latbins+latb;
+
+       if(llbin<0 || llbin>(nodes->file.latbins*nodes->file.lonbins))
+          continue;
+
+       index1=LookupNodeOffset(nodes,llbin);
+       index2=LookupNodeOffset(nodes,llbin+1);
+
+       for(item=index1;item<index2;item++)
+         {
+          Node *node=LookupNode(nodes,item,1);
+          double lat=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb)+off_to_latlong(node->latoffset));
+          double lon=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb)+off_to_latlong(node->lonoffset));
+
+          if(lat>latmin && lat<latmax && lon>lonmin && lon<lonmax)
+            {
+             Segment *segment;
+
+             print_node_osm(nodes,item);
+
+             segment=FirstSegment(segments,node,1);
+
+             while(segment)
+               {
+                double olat,olon;
+                index_t oitem=OtherNode(segment,item);
+
+                GetLatLong(nodes,oitem,&olat,&olon);
+
+                if(olat>latmin && olat<latmax && olon>lonmin && olon<lonmax)
+                   if(item>oitem)
+                      if(!option_no_super || IsNormalSegment(segment))
+                         print_segment_osm(segments,IndexSegment(segments,segment),ways);
+
+                segment=NextSegment(segments,segment,item);
+               }
+
+             if(IsTurnRestrictedNode(node))
+               {
+                index_t relindex=FindFirstTurnRelation1(relations,item);
+
+                while(relindex!=NO_RELATION)
+                  {
+                   print_turnrelation_osm(relations,relindex,segments,nodes);
+
+                   relindex=FindNextTurnRelation1(relations,relindex);
+                  }
+               }
+            }
+         }
+      }
 }
 
 
@@ -651,6 +708,9 @@ static void print_segment_osm(Segments *segments,index_t item,Ways *ways)
  if(way->type & Way_OneWay)
     printf("    <tag k='oneway' v='yes' />\n");
 
+ if(way->type & Way_Roundabout)
+    printf("    <tag k='roundabout' v='yes' />\n");
+
  printf("    <tag k='highway' v='%s' />\n",HighwayName(HIGHWAY(way->type)));
 
  if(IsNormalSegment(segment) && *name)
@@ -836,15 +896,17 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "  --data=<data-type>      * the type of data to select.\n"
             "\n"
             "  <data-type> can be selected from:\n"
-            "      junctions = segment count at each junction.\n"
-            "      super     = super-node and super-segments.\n"
-            "      oneway    = oneway segments.\n"
-            "      turns     = turn restrictions.\n"
-            "      speed     = speed limits.\n"
-            "      weight    = weight limits.\n"
-            "      height    = height limits.\n"
-            "      width     = width limits.\n"
-            "      length    = length limits.\n"
+            "      junctions   = segment count at each junction.\n"
+            "      super       = super-node and super-segments.\n"
+            "      oneway      = oneway segments.\n"
+            "      highway-*   = segments of the specified highway type.\n"
+            "      transport-* = segments allowing the specified transport type.\n"
+            "      turns       = turn restrictions.\n"
+            "      speed       = speed limits.\n"
+            "      weight      = weight limits.\n"
+            "      height      = height limits.\n"
+            "      width       = width limits.\n"
+            "      length      = length limits.\n"
             "\n"
             "--dump                    Dump selected contents of the database.\n"
             "  --node=<node>           * the node with the selected index.\n"
