@@ -3,7 +3,7 @@
 #
 # Part of the Routino routing software.
 #
-# This file Copyright 2008-2011 Andrew M. Bishop
+# This file Copyright 2008-2012 Andrew M. Bishop
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -30,8 +30,6 @@ use Time::HiRes qw(gettimeofday tv_interval);
 
 $t0 = [gettimeofday];
 
-# EDIT THIS to set the filename prefix for the routing database files.
-$data_prefix="";
 
 #
 # Fill in the default parameters using the ones above (don't use executable compiled in defaults)
@@ -43,12 +41,12 @@ sub FillInDefaults
 
    $params{transport}=$routino->{transport} if(!defined $params{transport});
 
-   my($transport)=$params{transport};
+   my $transport=$params{transport};
 
-   foreach $highway (keys %{$routino->{highways}})
+   foreach my $highway (keys %{$routino->{highways}})
      {
-      $key="highway-$highway";
-      $value=$routino->{profile_highway}->{$highway}->{$transport};
+      my $key="highway-$highway";
+      my $value=$routino->{profile_highway}->{$highway}->{$transport};
       $params{$key}=$value if(!defined $params{$key});
 
       $key="speed-$highway";
@@ -56,10 +54,10 @@ sub FillInDefaults
       $params{$key}=$value if(!defined $params{$key});
      }
 
-   foreach $property (keys %{$routino->{properties}})
+   foreach my $property (keys %{$routino->{properties}})
      {
-      $key="property-$property";
-      $value=$routino->{profile_property}->{$property}->{$transport};
+      my $key="property-$property";
+      my $value=$routino->{profile_property}->{$property}->{$transport};
       $params{$key}=$value if(!defined $params{$key});
      }
 
@@ -69,10 +67,10 @@ sub FillInDefaults
    $params{turns} =~ s/(true|on)/1/;
    $params{turns} =~ s/(false|off)/0/;
 
-   foreach $restriction (keys %{$routino->{restrictions}})
+   foreach my $restriction (keys %{$routino->{restrictions}})
      {
-      $key="$restriction";
-      $value=$routino->{profile_restrictions}->{$restriction}->{$transport};
+      my $key="$restriction";
+      my $value=$routino->{profile_restrictions}->{$restriction}->{$transport};
       $params{$key}=$value if(!defined $params{$key});
      }
 
@@ -90,9 +88,9 @@ sub RunRouter
 
    # Combine all of the parameters together
 
-   my($params)="--$optimise";
+   my $params="--$optimise";
 
-   foreach $key (keys %params)
+   foreach my $key (keys %params)
      {
       $params.=" --$key=$params{$key}";
      }
@@ -104,7 +102,16 @@ sub RunRouter
 
    # Create a unique output directory
 
-   chomp($uuid=`echo '$params' $$ | md5sum | cut -f1 '-d '`);
+   my $uuid;
+
+   if($^O eq "darwin")
+     {
+      chomp($uuid=`echo '$params' $$ | md5    | cut -f1 '-d '`);
+     }
+   else
+     {
+      chomp($uuid=`echo '$params' $$ | md5sum | cut -f1 '-d '`);
+     }
 
    mkdir $uuid;
    chmod 0775, $uuid;
@@ -118,22 +125,22 @@ sub RunRouter
 
    system "$bin_dir/$router_exe $params > router.log 2>&1";
 
-   (undef,undef,$cuser,$csystem) = times;
-   $time=sprintf "time: %.3f CPU / %.3f elapsed",$cuser+$csystem,tv_interval($t0);
+   my(undef,undef,$cuser,$csystem) = times;
+   my $time=sprintf "time: %.3f CPU / %.3f elapsed",$cuser+$csystem,tv_interval($t0);
 
-   $message="";
+   my $message="";
 
    if($? != 0)
      {
       $message=`tail -1 router.log`;
      }
 
-   $result="";
+   my $result="";
 
    if(-f "$optimise.txt")
      {
       $result=`tail -1 $optimise.txt`;
-      @result=split(/\t/,$result);
+      my @result=split(/\t/,$result);
       $result = $result[4]." , ".$result[5];
      }
 
@@ -175,10 +182,10 @@ sub ReturnOutput
 
    if($type eq "router") { $format="log" }
 
-   $suffix=$suffixes{$format};
-   $mime  =$mimetypes{$format};
+   my $suffix=$suffixes{$format};
+   my $mime  =$mimetypes{$format};
 
-   $file="$results_dir/$uuid/$type$suffix";
+   my $file="$results_dir/$uuid/$type$suffix";
 
    # Return the output
 
