@@ -3,7 +3,7 @@
 //
 // Part of the Routino routing software.
 //
-// This file Copyright 2008,2009 Andrew M. Bishop
+// This file Copyright 2008-2012 Andrew M. Bishop
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Affero General Public License as published by
@@ -24,16 +24,19 @@
 // Data types
 //
 
-var data_types={
-                junctions: true,
-                super: true,
-                oneway: true,
-                speed: true,
-                weight: true,
-                height: true,
-                width: true,
-                length: true
-               };
+var data_types=[
+                "junctions",
+                "super",
+                "oneway",
+                "highway",
+                "transport",
+                "turns",
+                "speed",
+                "weight",
+                "height",
+                "width",
+                "length"
+               ];
 
 
 //
@@ -64,34 +67,73 @@ var super_node_style,super_segment_style;
 
 
 //
-// Oneway styles
+// Oneway and turn restriction styles
 //
 
 var hex={0: "00", 1: "11",  2: "22",  3: "33",  4: "44",  5: "55",  6: "66",  7: "77",
          8: "88", 9: "99", 10: "AA", 11: "BB", 12: "CC", 13: "DD", 14: "EE", 15: "FF"};
 
+var turn_restriction_style;
 
-//
-// Map configuration
-//
+
+////////////////////////////////////////////////////////////////////////////////
+/////////////////////////////// Initialisation /////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
+
+// Process the URL query string and extract the arguments
+
+var legal={"^lon"  : "^[-0-9.]+$",
+           "^lat"  : "^[-0-9.]+$",
+           "^zoom" : "^[0-9]+$"};
+
+var args={};
+
+if(location.search.length>1)
+  {
+   var query,queries;
+
+   query=location.search.replace(/^\?/,"");
+   query=query.replace(/;/g,'&');
+   queries=query.split('&');
+
+   for(var i=0;i<queries.length;i++)
+     {
+      queries[i].match(/^([^=]+)(=(.*))?$/);
+
+      k=RegExp.$1;
+      v=unescape(RegExp.$3);
+
+      for(var l in legal)
+        {
+         if(k.match(RegExp(l)) && v.match(RegExp(legal[l])))
+            args[k]=v;
+        }
+     }
+  }
+
+
+////////////////////////////////////////////////////////////////////////////////
+///////////////////////////////// Map handling /////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
 
 var map;
-var layerMapOSM, layerVectors, layerBoxes;
+var layerMap=[], layerVectors, layerBoxes;
 var epsg4326, epsg900913;
 
 var box;
 
-
 // 
 // Initialise the 'map' object
 //
 
-function map_init(lat,lon,zoom)
+function map_init()
 {
- var data;
+ lon =args["lon"];
+ lat =args["lat"];
+ zoom=args["zoom"];
 
- for(data in data_types)
-    hideshow_hide(data);
+ // Map properties (North/South and East/West limits and zoom in/out limits) are now in mapprops.js
+ // Map URLs are now in mapprops.js
 
  //
  // Create the map
@@ -100,11 +142,6 @@ function map_init(lat,lon,zoom)
  epsg4326=new OpenLayers.Projection("EPSG:4326");
  epsg900913=new OpenLayers.Projection("EPSG:900913");
 
- // UK coordinate range: West -11.0, South 49.5, East 2.0, North 61.0
-
- // EDIT THIS to change the visible map boundary.
- var mapbounds=new OpenLayers.Bounds(-11.0,49.5,2.0,61.0).transform(epsg4326,epsg900913);
-
  map = new OpenLayers.Map ("map",
                            {
                             controls:[
@@ -117,47 +154,43 @@ function map_init(lat,lon,zoom)
                             projection: epsg900913,
                             displayProjection: epsg4326,
 
-                            // EDIT THIS to set the minimum zoom level
-                            minZoomLevel: 4,
+                            minZoomLevel: mapprops.zoomout,
+                            numZoomLevels: mapprops.zoomin-mapprops.zoomout+1,
+                            maxResolution: 156543.03390625 / Math.pow(2,mapprops.zoomout),
 
-                            // EDIT THIS to set the number of zoom levels
-                            numZoomLevels: 12, // zoom levels 4-15 inclusive
+                            // These two lines are not needed with OpenLayers 2.12
+                            units: "m",
+                            maxExtent:        new OpenLayers.Bounds(-20037508.34, -20037508.34, 20037508.34, 20037508.34),
 
-                            // EDIT THIS if you change the minimum zoom level above
-                            maxResolution: 156543.0339 / Math.pow(2,4), // Math.pow(2,minZoomLevel)
-
-                            maxExtent: new OpenLayers.Bounds(-20037508.34, -20037508.34, 20037508.34, 20037508.34),
-
-                            restrictedExtent: mapbounds,
-
-                            units: "m"
+                            restrictedExtent: new OpenLayers.Bounds(mapprops.westedge,mapprops.southedge,mapprops.eastedge,mapprops.northedge).transform(epsg4326,epsg900913)
                            });
 
- map.events.register("moveend", map, mapMoved);
+ // Add map tile layers
 
- // Add a map tile layer (OpenStreetMap tiles, direct access)
+ for(var l=0;l < mapprops.mapdata.length;l++)
+   {
+    layerMap[l] = new OpenLayers.Layer.TMS(mapprops.mapdata[l].label,
+                                           mapprops.mapdata[l].baseurl,
+                                           {
+                                            emptyUrl: mapprops.mapdata[l].errorurl,
+                                            type: 'png',
+                                            getURL: limitedUrl,
+                                            displayOutsideMaxExtent: true,
+                                            buffer: 1
+                                           });
+    map.addLayer(layerMap[l]);
+   }
 
- layerMapOSM = new OpenLayers.Layer.TMS("Original OSM map",
-                                        "http://tile.openstreetmap.org/",
-                                        {
-                                         emptyUrl: "http://openstreetmap.org/openlayers/img/404.png",
-                                         type: 'png',
-                                         getURL: limitedUrl,
-                                         displayOutsideMaxExtent: true,
-                                         buffer: 1
-                                        });
- map.addLayer(layerMapOSM);
-
- // Get a URL for the tile; limited to mapbounds.
+ // Get a URL for the tile; limited to map restricted extent.
 
  function limitedUrl(bounds)
  {
   var z = map.getZoom() + map.minZoomLevel;
 
-  if (z>7 && (bounds.right  < mapbounds.left ||
-              bounds.left   > mapbounds.right ||
-              bounds.top    < mapbounds.bottom ||
-              bounds.bottom > mapbounds.top))
+  if (z>=7 && (bounds.right  < map.restrictedExtent.left ||
+               bounds.left   > map.restrictedExtent.right ||
+               bounds.top    < map.restrictedExtent.bottom ||
+               bounds.bottom > map.restrictedExtent.top))
      return this.emptyUrl;
 
   var res = map.getResolution();
@@ -178,13 +211,14 @@ function map_init(lat,lon,zoom)
  layerVectors = new OpenLayers.Layer.Vector("Markers");
  map.addLayer(layerVectors);
 
- var colour;
- for(colour in junction_colours)
+ for(var colour in junction_colours)
     junction_styles[colour]=new OpenLayers.Style({},{stroke: false, pointRadius: 2,fillColor: junction_colours[colour]});
 
  super_node_style   =new OpenLayers.Style({},{stroke: false, pointRadius: 3,fillColor  : "#FF0000"});
  super_segment_style=new OpenLayers.Style({},{fill: false  , strokeWidth: 2,strokeColor: "#FF0000"});
 
+ turn_restriction_style=new OpenLayers.Style({},{fill: false, strokeWidth: 2,strokeColor: "#FF0000"});
+
  // Add a boxes layer
 
  layerBoxes = new OpenLayers.Layer.Boxes("Boundary");
@@ -194,14 +228,24 @@ function map_init(lat,lon,zoom)
 
  // Set the map centre to the limited range specified
 
- map.setCenter(mapbounds.getCenterLonLat(), map.getZoomForExtent(mapbounds,true));
+ map.setCenter(map.restrictedExtent.getCenterLonLat(), map.getZoomForExtent(map.restrictedExtent,true));
  map.maxResolution = map.getResolution();
 
  // Move the map
 
- if(lon != 'lon' && lat != 'lat' && zoom != 'zoom')
+ if(lon != undefined && lat != undefined && zoom != undefined)
    {
-    var lonlat = new OpenLayers.LonLat(lon,lat).transform(epsg4326,map.getProjectionObject());
+    if(lon<mapprops.westedge) lon=mapprops.westedge;
+    if(lon>mapprops.eastedge) lon=mapprops.eastedge;
+
+    if(lat<mapprops.southedge) lat=mapprops.southedge;
+    if(lat>mapprops.northedge) lat=mapprops.northedge;
+
+    if(zoom<mapprops.zoomout) zoom=mapprops.zoomout;
+    if(zoom>mapprops.zoomin)  zoom=mapprops.zoomin;
+
+    var lonlat = new OpenLayers.LonLat(lon,lat);
+    lonlat.transform(epsg4326,epsg900913);
 
     map.moveTo(lonlat,zoom-map.minZoomLevel);
    }
@@ -209,26 +253,95 @@ function map_init(lat,lon,zoom)
 
 
 //
-// Map has moved
+// Format a number in printf("%.5f") format.
 //
 
-function mapMoved()
+function format5f(number)
 {
- var centre = map.getCenter().clone();
+ var newnumber=Math.floor(number*100000+0.5);
+ var delta=0;
 
- var lonlat = centre.transform(map.getProjectionObject(),epsg4326);
+ if(newnumber>=0 && newnumber<100000) delta= 100000;
+ if(newnumber<0 && newnumber>-100000) delta=-100000;
 
- var zoom = this.getZoom() + map.minZoomLevel;
+ var string=String(newnumber+delta);
 
- var router_url=document.getElementById("router_url");
- var link_url  =document.getElementById("link_url");
- var edit_url  =document.getElementById("edit_url");
+ var intpart =string.substring(0,string.length-5);
+ var fracpart=string.substring(string.length-5,string.length);
 
- var args="lat=" + lonlat.lat + ";lon=" + lonlat.lon + ";zoom=" + zoom;
+ if(delta>0) intpart="0";
+ if(delta<0) intpart="-0";
 
- router_url.href="customrouter.cgi?" + args;
- link_url.href="customvisualiser.cgi?" + args;
- edit_url.href="http://www.openstreetmap.org/edit?" + args;
+ return(intpart + "." + fracpart);
+}
+
+
+//
+// Build a set of URL arguments for the map location
+//
+
+function buildMapArguments()
+{
+ var lonlat = map.getCenter().clone();
+ lonlat.transform(epsg900913,epsg4326);
+
+ var zoom = map.getZoom() + map.minZoomLevel;
+
+ return "lat=" + format5f(lonlat.lat) + ";lon=" + format5f(lonlat.lon) + ";zoom=" + zoom;
+}
+
+
+//
+// Update a URL
+//
+
+function updateURL(element)
+{
+ if(element.id == "permalink_url")
+    element.href=location.pathname + "?" + buildMapArguments();
+
+ if(element.id == "router_url")
+    element.href="router.html" + "?" + buildMapArguments();
+
+ if(element.id == "edit_url")
+    element.href="http://www.openstreetmap.org/edit" + "?" + buildMapArguments();
+
+ if(element.id.match(/^lang_([a-zA-Z-]+)_url$/))
+    element.href="visualiser.html" + "." + RegExp.$1 + "?" + buildMapArguments();
+}
+
+
+////////////////////////////////////////////////////////////////////////////////
+/////////////////////////////// Server handling ////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
+
+//
+// Display the status
+//
+
+function displayStatus(type,subtype,content)
+{
+ var child=document.getElementById("result_status").firstChild;
+
+ do
+   {
+    if(child.id != undefined)
+       child.style.display="none";
+
+    child=child.nextSibling;
+   }
+ while(child != undefined);
+
+ var chosen_status=document.getElementById("result_status_" + type);
+
+ chosen_status.style.display="";
+
+ if(subtype != null)
+   {
+    var format_status=document.getElementById("result_status_" + subtype).innerHTML;
+
+    chosen_status.innerHTML=format_status.replace('#',String(content));
+   }
 }
 
 
@@ -240,22 +353,18 @@ function displayStatistics()
 {
  // Use AJAX to get the statistics
 
- OpenLayers.loadURL("statistics.cgi",null,null,runStatisticsSuccess);
+ OpenLayers.Request.GET({url: "statistics.cgi", success: runStatisticsSuccess});
 }
 
 
 //
-// Success in running router.
+// Success in running data statistics generation.
 //
 
 function runStatisticsSuccess(response)
 {
- var statistics_data=document.getElementById("statistics_data");
- var statistics_link=document.getElementById("statistics_link");
-
- statistics_data.innerHTML="<pre>" + response.responseText + "</pre>";
-
- statistics_link.style.display="none";
+ document.getElementById("statistics_data").innerHTML="<pre>" + response.responseText + "</pre>";
+ document.getElementById("statistics_link").style.display="none";
 }
 
 
@@ -265,10 +374,8 @@ function runStatisticsSuccess(response)
 
 function displayData(datatype)
 {
- var data;
-
- for(data in data_types)
-    hideshow_hide(data);
+ for(var data in data_types)
+    hideshow_hide(data_types[data]);
 
  if(datatype != "")
     hideshow_show(datatype);
@@ -283,8 +390,7 @@ function displayData(datatype)
 
  // Print the status
 
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "No data displayed";
+ displayStatus("no_data");
 
  // Return if just here to clear the data
 
@@ -304,29 +410,44 @@ function displayData(datatype)
  url=url + ";latmax=" + mapbounds.top;
  url=url + ";data=" + datatype;
 
- // Print the status
-
- div_status.innerHTML = "Fetching " + datatype + " data ...";
-
  // Use AJAX to get the data
 
  switch(datatype)
    {
    case 'junctions':
-    OpenLayers.loadURL(url,null,null,runJunctionsSuccess,runFailure);
+    OpenLayers.Request.GET({url: url, success: runJunctionsSuccess, failure: runFailure});
     break;
    case 'super':
-    OpenLayers.loadURL(url,null,null,runSuperSuccess,runFailure);
+    OpenLayers.Request.GET({url: url, success: runSuperSuccess, faliure: runFailure});
     break;
    case 'oneway':
-    OpenLayers.loadURL(url,null,null,runOnewaySuccess,runFailure);
+    OpenLayers.Request.GET({url: url, success: runOnewaySuccess, failure: runFailure});
+    break;
+   case 'highway':
+    var highways=document.forms["highways"].elements["highway"];
+    for(var h in highways)
+       if(highways[h].checked)
+          highway=highways[h].value;
+    url+="-" + highway;
+    OpenLayers.Request.GET({url: url, success: runHighwaySuccess, falure: runFailure});
+    break;
+   case 'transport':
+    var transports=document.forms["transports"].elements["transport"];
+    for(var t in transports)
+       if(transports[t].checked)
+          transport=transports[t].value;
+    url+="-" + transport;
+    OpenLayers.Request.GET({url: url, success: runTransportSuccess, failure: runFailure});
+    break;
+   case 'turns':
+    OpenLayers.Request.GET({url: url, success: runTurnsSuccess, failure: runFailure});
     break;
    case 'speed':
    case 'weight':
    case 'height':
    case 'width':
    case 'length':
-    OpenLayers.loadURL(url,null,null,runLimitSuccess,runFailure);
+    OpenLayers.Request.GET({url: url, success: runLimitSuccess, failure: runFailure});
     break;
    }
 }
@@ -340,12 +461,9 @@ function runJunctionsSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "Processing " + (lines.length-2) + " junctions ...";
-
  var features=[];
 
- for(line in lines)
+ for(var line=0;line<lines.length;line++)
    {
     var words=lines[line].split(' ');
 
@@ -356,7 +474,7 @@ function runJunctionsSuccess(response)
        var lat2=words[2];
        var lon2=words[3];
 
-       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,map.getProjectionObject());
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
 
        box = new OpenLayers.Marker.Box(bounds);
 
@@ -373,13 +491,12 @@ function runJunctionsSuccess(response)
        var point = new OpenLayers.Geometry.Point(lonlat.lon,lonlat.lat);
 
        features.push(new OpenLayers.Feature.Vector(point,{},junction_styles[count]));
-
       }
    }
 
  layerVectors.addFeatures(features);
 
- div_status.innerHTML = "Processed " + (lines.length-2) + " junctions";
+ displayStatus("data","junctions",lines.length-2);
 }
 
 
@@ -391,14 +508,11 @@ function runSuperSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "Processing " + (lines.length-2) + " super-nodes/segments ...";
-
  var features=[];
 
  var nodepoint;
 
- for(line in lines)
+ for(var line=0;line<lines.length;line++)
    {
     var words=lines[line].split(' ');
 
@@ -409,7 +523,7 @@ function runSuperSuccess(response)
        var lat2=words[2];
        var lon2=words[3];
 
-       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,map.getProjectionObject());
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
 
        box = new OpenLayers.Marker.Box(bounds);
 
@@ -433,16 +547,16 @@ function runSuperSuccess(response)
          }
        else
          {
-          var line = new OpenLayers.Geometry.LineString([nodepoint,point]);
+          var segment = new OpenLayers.Geometry.LineString([nodepoint,point]);
 
-          features.push(new OpenLayers.Feature.Vector(line,{},super_segment_style));
+          features.push(new OpenLayers.Feature.Vector(segment,{},super_segment_style));
          }
       }
    }
 
  layerVectors.addFeatures(features);
 
- div_status.innerHTML = "Processed " + (lines.length-2) + " super-nodes/segments";
+ displayStatus("data","super",lines.length-2);
 }
 
 
@@ -454,12 +568,9 @@ function runOnewaySuccess(response)
 {
  var lines=response.responseText.split('\n');
 
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "Processing " + (lines.length-2) + " oneway segments ...";
-
  var features=[];
 
- for(line in lines)
+ for(var line=0;line<lines.length;line++)
    {
     var words=lines[line].split(' ');
 
@@ -470,7 +581,7 @@ function runOnewaySuccess(response)
        var lat2=words[2];
        var lon2=words[3];
 
-       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,map.getProjectionObject());
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
 
        box = new OpenLayers.Marker.Box(bounds);
 
@@ -497,7 +608,7 @@ function runOnewaySuccess(response)
        var point3 = new OpenLayers.Geometry.Point(lonlat1.lon+dlat/dist,lonlat1.lat-dlon/dist);
        var point4 = new OpenLayers.Geometry.Point(lonlat1.lon-dlat/dist,lonlat1.lat+dlon/dist);
 
-       var line = new OpenLayers.Geometry.LineString([point2,point3,point4,point2]);
+       var segment = new OpenLayers.Geometry.LineString([point2,point3,point4,point2]);
 
        var r=Math.round(7.5+7.9*Math.cos(ang));
        var g=Math.round(7.5+7.9*Math.cos(ang+2.0943951));
@@ -506,13 +617,173 @@ function runOnewaySuccess(response)
 
        var style=new OpenLayers.Style({},{strokeWidth: 2,strokeColor: colour});
 
-       features.push(new OpenLayers.Feature.Vector(line,{},style));
+       features.push(new OpenLayers.Feature.Vector(segment,{},style));
       }
    }
 
  layerVectors.addFeatures(features);
 
- div_status.innerHTML = "Processed " + (lines.length-2) + " oneway segments";
+ displayStatus("data","oneway",lines.length-2);
+}
+
+
+//
+// Success in getting the highway data
+//
+
+function runHighwaySuccess(response)
+{
+ var lines=response.responseText.split('\n');
+
+ var features=[];
+
+ for(var line=0;line<lines.length;line++)
+   {
+    var words=lines[line].split(' ');
+
+    if(line == 0)
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
+
+       box = new OpenLayers.Marker.Box(bounds);
+
+       layerBoxes.addMarker(box);
+      }
+    else if(words[0] != "")
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
+       var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
+
+       var point1 = new OpenLayers.Geometry.Point(lonlat1.lon,lonlat1.lat);
+       var point2 = new OpenLayers.Geometry.Point(lonlat2.lon,lonlat2.lat);
+
+       var segment = new OpenLayers.Geometry.LineString([point1,point2]);
+
+       features.push(new OpenLayers.Feature.Vector(segment,{},super_segment_style));
+      }
+   }
+
+ layerVectors.addFeatures(features);
+
+ displayStatus("data","highway",lines.length-2);
+}
+
+
+//
+// Success in getting the transport data
+//
+
+function runTransportSuccess(response)
+{
+ var lines=response.responseText.split('\n');
+
+ var features=[];
+
+ for(var line=0;line<lines.length;line++)
+   {
+    var words=lines[line].split(' ');
+
+    if(line == 0)
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
+
+       box = new OpenLayers.Marker.Box(bounds);
+
+       layerBoxes.addMarker(box);
+      }
+    else if(words[0] != "")
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
+       var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
+
+       var point1 = new OpenLayers.Geometry.Point(lonlat1.lon,lonlat1.lat);
+       var point2 = new OpenLayers.Geometry.Point(lonlat2.lon,lonlat2.lat);
+
+       var segment = new OpenLayers.Geometry.LineString([point1,point2]);
+
+       features.push(new OpenLayers.Feature.Vector(segment,{},super_segment_style));
+      }
+   }
+
+ layerVectors.addFeatures(features);
+
+ displayStatus("data","transport",lines.length-2);
+}
+
+
+//
+// Success in getting the turn restrictions data
+//
+
+function runTurnsSuccess(response)
+{
+ var lines=response.responseText.split('\n');
+
+ var features=[];
+
+ for(var line=0;line<lines.length;line++)
+   {
+    var words=lines[line].split(' ');
+
+    if(line == 0)
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
+
+       box = new OpenLayers.Marker.Box(bounds);
+
+       layerBoxes.addMarker(box);
+      }
+    else if(words[0] != "")
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+       var lat3=words[4];
+       var lon3=words[5];
+
+       var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
+       var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
+       var lonlat3= new OpenLayers.LonLat(lon3,lat3).transform(epsg4326,epsg900913);
+
+       var point1 = new OpenLayers.Geometry.Point(lonlat1.lon,lonlat1.lat);
+       var point2 = new OpenLayers.Geometry.Point(lonlat2.lon,lonlat2.lat);
+       var point3 = new OpenLayers.Geometry.Point(lonlat3.lon,lonlat3.lat);
+
+       var segments = new OpenLayers.Geometry.LineString([point1,point2,point3]);
+
+       features.push(new OpenLayers.Feature.Vector(segments,{},turn_restriction_style));
+      }
+   }
+
+ layerVectors.addFeatures(features);
+
+ displayStatus("data","turns",lines.length-2);
 }
 
 
@@ -524,14 +795,11 @@ function runLimitSuccess(response)
 {
  var lines=response.responseText.split('\n');
 
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "Processing " + (lines.length-2) + " limits ...";
-
  var features=[];
 
  var nodelonlat;
 
- for(line in lines)
+ for(var line=0;line<lines.length;line++)
    {
     var words=lines[line].split(' ');
 
@@ -542,7 +810,7 @@ function runLimitSuccess(response)
        var lat2=words[2];
        var lon2=words[3];
 
-       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,map.getProjectionObject());
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
 
        box = new OpenLayers.Marker.Box(bounds);
 
@@ -583,7 +851,7 @@ function runLimitSuccess(response)
 
  layerVectors.addFeatures(features);
 
- div_status.innerHTML = "Processed " + (lines.length-2) + " limits";
+ displayStatus("data","limit",lines.length-2);
 }
 
 
@@ -593,8 +861,5 @@ function runLimitSuccess(response)
 
 function runFailure(response)
 {
- var div_status=document.getElementById("result_status");
- div_status.innerHTML = "Failed to get visualiser data!";
-
- window.alert("Failed to get visualiser data!\n" + response.statusText);
+ displayStatus("error");
 }
